@@ -81,41 +81,47 @@ namespace common {
 #if defined(_MSC_VER)
 
 // Windows CUDA doesn't have __assert_fail.
-#define KERNEL_CHECK(cond)                                                     \
-  do {                                                                         \
-    if (XGBOOST_EXPECT(!(cond), false)) {                                      \
-      asm("trap;");                                                            \
-    }                                                                          \
+#define CUDA_KERNEL_CHECK(cond)           \
+  do {                                    \
+    if (XGBOOST_EXPECT(!(cond), false)) { \
+      asm("trap;");                       \
+    }                                     \
   } while (0)
 
 #else  // defined(_MSC_VER)
 
 #define __ASSERT_STR_HELPER(x) #x
 
-#define KERNEL_CHECK(cond)                                                     \
-  (XGBOOST_EXPECT((cond), true)                                                \
-       ? static_cast<void>(0)                                                  \
-       : __assert_fail(__ASSERT_STR_HELPER((cond)), __FILE__, __LINE__,        \
-                       __PRETTY_FUNCTION__))
+#define CUDA_KERNEL_CHECK(cond) \
+  (XGBOOST_EXPECT((cond), true) \
+       ? static_cast<void>(0)   \
+       : __assert_fail(__ASSERT_STR_HELPER((cond)), __FILE__, __LINE__, __PRETTY_FUNCTION__))
 
 #endif  // defined(_MSC_VER)
 
+#define KERNEL_CHECK CUDA_KERNEL_CHECK
+
 #define SPAN_CHECK KERNEL_CHECK
 
-#else  // not CUDA
+#else  // ------------------------------ not CUDA ----------------------------
 
-#define KERNEL_CHECK(cond)                                                     \
-  (XGBOOST_EXPECT((cond), true) ? static_cast<void>(0) : std::terminate())
+#if defined(XGBOOST_STRICT_R_MODE) && XGBOOST_STRICT_R_MODE == 1
+
+#define KERNEL_CHECK(cond)
 
 #define SPAN_CHECK(cond) KERNEL_CHECK(cond)
 
+#else
+
+#define KERNEL_CHECK(cond) (XGBOOST_EXPECT((cond), true) ? static_cast<void>(0) : std::terminate())
+
+#define SPAN_CHECK(cond) KERNEL_CHECK(cond)
+
+#endif  // defined(XGBOOST_STRICT_R_MODE)
+
 #endif  // __CUDA_ARCH__
 
-#if defined(__CUDA_ARCH__)
-#define SPAN_LT(lhs, rhs) KERNEL_CHECK((lhs) < (rhs))
-#else
-#define SPAN_LT(lhs, rhs) KERNEL_CHECK((lhs) < (rhs))
-#endif  // defined(__CUDA_ARCH__)
+#define SPAN_LT(lhs, rhs) SPAN_CHECK((lhs) < (rhs))
 
 namespace detail {
 /*!
@@ -423,10 +429,10 @@ class Span {
   using pointer = T*;                                   // NOLINT
   using reference = T&;                                 // NOLINT
 
-  using iterator = detail::SpanIterator<Span<T, Extent>, false>;             // NOLINT
-  using const_iterator = const detail::SpanIterator<Span<T, Extent>, true>;  // NOLINT
-  using reverse_iterator = detail::SpanIterator<Span<T, Extent>, false>;     // NOLINT
-  using const_reverse_iterator = const detail::SpanIterator<Span<T, Extent>, true>;  // NOLINT
+  using iterator = detail::SpanIterator<Span<T, Extent>, false>;               // NOLINT
+  using const_iterator = const detail::SpanIterator<Span<T, Extent>, true>;    // NOLINT
+  using reverse_iterator = std::reverse_iterator<iterator>;                    // NOLINT
+  using const_reverse_iterator = const std::reverse_iterator<const_iterator>;  // NOLINT
 
   // constructors
   constexpr Span() __span_noexcept = default;
@@ -504,11 +510,11 @@ class Span {
     return {this, size()};
   }
 
-  XGBOOST_DEVICE constexpr reverse_iterator rbegin() const __span_noexcept {  // NOLINT
+  constexpr reverse_iterator rbegin() const __span_noexcept {  // NOLINT
     return reverse_iterator{end()};
   }
 
-  XGBOOST_DEVICE constexpr reverse_iterator rend() const __span_noexcept {    // NOLINT
+  constexpr reverse_iterator rend() const __span_noexcept {  // NOLINT
     return reverse_iterator{begin()};
   }
 
@@ -662,7 +668,6 @@ XGBOOST_DEVICE auto as_writable_bytes(Span<T, E> s) __span_noexcept ->  // NOLIN
     Span<byte, detail::ExtentAsBytesValue<T, E>::value> {
   return {reinterpret_cast<byte*>(s.data()), s.size_bytes()};
 }
-
 }  // namespace common
 }  // namespace xgboost
 
