@@ -1,5 +1,5 @@
 /*
- Copyright (c) 2014 by Contributors
+ Copyright (c) 2014,2021 by Contributors
 
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
@@ -17,8 +17,9 @@
 package ml.dmlc.xgboost4j.scala
 
 import _root_.scala.collection.JavaConverters._
+
 import ml.dmlc.xgboost4j.LabeledPoint
-import ml.dmlc.xgboost4j.java.{DMatrix => JDMatrix, DataBatch, XGBoostError}
+import ml.dmlc.xgboost4j.java.{Column, ColumnBatch, DataBatch, XGBoostError, DMatrix => JDMatrix}
 
 class DMatrix private[scala](private[scala] val jDMatrix: JDMatrix) {
   /**
@@ -72,6 +73,18 @@ class DMatrix private[scala](private[scala] val jDMatrix: JDMatrix) {
     this(new JDMatrix(headers, indices, data, st, shapeParam))
   }
 
+  /**
+   * Create the normal DMatrix from column array interface
+   * @param columnBatch the XGBoost ColumnBatch to provide the cuda array interface
+   *                    of feature columns
+   * @param missing missing value
+   * @param nthread threads number
+   */
+  @throws(classOf[XGBoostError])
+  def this(columnBatch: ColumnBatch, missing: Float, nthread: Int) {
+    this(new JDMatrix(columnBatch, missing, nthread))
+  }
+
   /**
    * create DMatrix from dense matrix
    *
@@ -150,6 +163,30 @@ class DMatrix private[scala](private[scala] val jDMatrix: JDMatrix) {
     jDMatrix.setGroup(group)
   }
 
+  /**
+   * Set label of DMatrix from cuda array interface
+   */
+  @throws(classOf[XGBoostError])
+  def setLabel(column: Column): Unit = {
+    jDMatrix.setLabel(column)
+  }
+
+  /**
+   * set weight of dmatrix from column array interface
+   */
+  @throws(classOf[XGBoostError])
+  def setWeight(column: Column): Unit = {
+    jDMatrix.setWeight(column)
+  }
+
+  /**
+   * set base margin of dmatrix from column array interface
+   */
+  @throws(classOf[XGBoostError])
+  def setBaseMargin(column: Column): Unit = {
+    jDMatrix.setBaseMargin(column)
+  }
+
   /**
    * Get group sizes of DMatrix (used for ranking)
    */
