@@ -10,13 +10,11 @@ from abc import ABC
 import collections
 import os
 import pickle
-from typing import Callable, List, Optional, Union, Dict, Tuple, TypeVar, cast
-from typing import Sequence
+from typing import Callable, List, Optional, Union, Dict, Tuple, TypeVar, cast, Sequence, Any
 import numpy
 
-from . import rabit
+from . import collective
 from .core import Booster, DMatrix, XGBoostError, _get_booster_layer_trees
-from .compat import STRING_TYPES
 
 
 __all__ = [
@@ -25,11 +23,14 @@ __all__ = [
     "EarlyStopping",
     "EvaluationMonitor",
     "TrainingCheckPoint",
+    "CallbackContainer"
 ]
 
 _Score = Union[float, Tuple[float, float]]
 _ScoreList = Union[List[float], List[Tuple[float, float]]]
 
+_Model = Any  # real type is Union[Booster, CVPack]; need more work
+
 
 # pylint: disable=unused-argument
 class TrainingCallback(ABC):
@@ -44,19 +45,19 @@ class TrainingCallback(ABC):
     def __init__(self) -> None:
         pass
 
-    def before_training(self, model):
+    def before_training(self, model: _Model) -> _Model:
         '''Run before training starts.'''
         return model
 
-    def after_training(self, model):
+    def after_training(self, model: _Model) -> _Model:
         '''Run after training is finished.'''
         return model
 
-    def before_iteration(self, model, epoch: int, evals_log: EvalsLog) -> bool:
+    def before_iteration(self, model: _Model, epoch: int, evals_log: EvalsLog) -> bool:
         '''Run before each iteration.  Return True when training should stop.'''
         return False
 
-    def after_iteration(self, model, epoch: int, evals_log: EvalsLog) -> bool:
+    def after_iteration(self, model: _Model, epoch: int, evals_log: EvalsLog) -> bool:
         '''Run after each iteration.  Return True when training should stop.'''
         return False
 
@@ -82,7 +83,7 @@ def _aggcv(rlist: List[str]) -> List[Tuple[str, float, float]]:
     results = []
     for (_, name), s in sorted(cvmap.items(), key=lambda x: x[0][0]):
         as_arr = numpy.array(s)
-        if not isinstance(msg, STRING_TYPES):
+        if not isinstance(msg, str):
             msg = msg.decode()
         mean, std = numpy.mean(as_arr), numpy.std(as_arr)
         results.extend([(name, mean, std)])
@@ -99,7 +100,7 @@ def _allreduce_metric(score: _ART) -> _ART:
     as final result.
 
     '''
-    world = rabit.get_world_size()
+    world = collective.get_world_size()
     assert world != 0
     if world == 1:
         return score
@@ -107,7 +108,7 @@ def _allreduce_metric(score: _ART) -> _ART:
         raise ValueError(
             'xgboost.cv function should not be used in distributed environment.')
     arr = numpy.array([score])
-    arr = rabit.allreduce(arr, rabit.Op.SUM) / world
+    arr = collective.allreduce(arr, collective.Op.SUM) / world
     return arr[0]
 
 
@@ -141,7 +142,7 @@ class CallbackContainer:
         if self.is_cv:
             self.aggregated_cv = None
 
-    def before_training(self, model):
+    def before_training(self, model: _Model) -> _Model:
         '''Function called before training.'''
         for c in self.callbacks:
             model = c.before_training(model=model)
@@ -152,7 +153,7 @@ class CallbackContainer:
                 assert isinstance(model, Booster), msg
         return model
 
-    def after_training(self, model):
+    def after_training(self, model: _Model) -> _Model:
         '''Function called after training.'''
         for c in self.callbacks:
             model = c.after_training(model=model)
@@ -183,7 +184,7 @@ class CallbackContainer:
         return model
 
     def before_iteration(
-        self, model, epoch: int, dtrain: DMatrix, evals: List[Tuple[DMatrix, str]]
+        self, model: _Model, epoch: int, dtrain: DMatrix, evals: Optional[List[Tuple[DMatrix, str]]]
     ) -> bool:
         '''Function called before training iteration.'''
         return any(c.before_iteration(model, epoch, self.history)
@@ -221,7 +222,7 @@ class CallbackContainer:
 
     def after_iteration(
         self,
-        model,
+        model: _Model,
         epoch: int,
         dtrain: DMatrix,
         evals: Optional[List[Tuple[DMatrix, str]]],
@@ -277,7 +278,7 @@ class LearningRateScheduler(TrainingCallback):
         super().__init__()
 
     def after_iteration(
-        self, model, epoch: int, evals_log: TrainingCallback.EvalsLog
+        self, model: _Model, epoch: int, evals_log: TrainingCallback.EvalsLog
     ) -> bool:
         model.set_param("learning_rate", self.learning_rates(epoch))
         return False
@@ -345,12 +346,12 @@ class EarlyStopping(TrainingCallback):
         self.starting_round: int = 0
         super().__init__()
 
-    def before_training(self, model):
+    def before_training(self, model: _Model) -> _Model:
         self.starting_round = model.num_boosted_rounds()
         return model
 
     def _update_rounds(
-        self, score: _Score, name: str, metric: str, model, epoch: int
+        self, score: _Score, name: str, metric: str, model: _Model, epoch: int
     ) -> bool:
         def get_s(x: _Score) -> float:
             """get score if it's cross validation history."""
@@ -404,7 +405,7 @@ class EarlyStopping(TrainingCallback):
             return True
         return False
 
-    def after_iteration(self, model, epoch: int,
+    def after_iteration(self, model: _Model, epoch: int,
                         evals_log: TrainingCallback.EvalsLog) -> bool:
         epoch += self.starting_round  # training continuation
         msg = 'Must have at least 1 validation dataset for early stopping.'
@@ -432,7 +433,7 @@ class EarlyStopping(TrainingCallback):
         score = data_log[metric_name][-1]
         return self._update_rounds(score, data_name, metric_name, model, epoch)
 
-    def after_training(self, model):
+    def after_training(self, model: _Model) -> _Model:
         try:
             if self.save_best:
                 model = model[: int(model.attr("best_iteration")) + 1]
@@ -478,13 +479,13 @@ class EvaluationMonitor(TrainingCallback):
             msg = f"\t{data + '-' + metric}:{score:.5f}"
         return msg
 
-    def after_iteration(self, model, epoch: int,
+    def after_iteration(self, model: _Model, epoch: int,
                         evals_log: TrainingCallback.EvalsLog) -> bool:
         if not evals_log:
             return False
 
         msg: str = f'[{epoch}]'
-        if rabit.get_rank() == self.printer_rank:
+        if collective.get_rank() == self.printer_rank:
             for data, metric in evals_log.items():
                 for metric_name, log in metric.items():
                     stdv: Optional[float] = None
@@ -497,16 +498,16 @@ class EvaluationMonitor(TrainingCallback):
             msg += '\n'
 
             if (epoch % self.period) == 0 or self.period == 1:
-                rabit.tracker_print(msg)
+                collective.communicator_print(msg)
                 self._latest = None
             else:
                 # There is skipped message
                 self._latest = msg
         return False
 
-    def after_training(self, model):
-        if rabit.get_rank() == self.printer_rank and self._latest is not None:
-            rabit.tracker_print(self._latest)
+    def after_training(self, model: _Model) -> _Model:
+        if collective.get_rank() == self.printer_rank and self._latest is not None:
+            collective.communicator_print(self._latest)
         return model
 
 
@@ -545,13 +546,13 @@ class TrainingCheckPoint(TrainingCallback):
         self._epoch = 0
         super().__init__()
 
-    def after_iteration(self, model, epoch: int,
+    def after_iteration(self, model: _Model, epoch: int,
                         evals_log: TrainingCallback.EvalsLog) -> bool:
         if self._epoch == self._iterations:
             path = os.path.join(self._path, self._name + '_' + str(epoch) +
                                 ('.pkl' if self._as_pickle else '.json'))
             self._epoch = 0
-            if rabit.get_rank() == 0:
+            if collective.get_rank() == 0:
                 if self._as_pickle:
                     with open(path, 'wb') as fd:
                         pickle.dump(model, fd)
