@@ -7,20 +7,21 @@
 #ifndef XGBOOST_COMMON_RANDOM_H_
 #define XGBOOST_COMMON_RANDOM_H_
 
-#include <rabit/rabit.h>
 #include <xgboost/logging.h>
+
 #include <algorithm>
 #include <functional>
-#include <vector>
 #include <limits>
 #include <map>
 #include <memory>
 #include <numeric>
 #include <random>
 #include <utility>
+#include <vector>
 
-#include "xgboost/host_device_vector.h"
+#include "../collective/communicator-inl.h"
 #include "common.h"
+#include "xgboost/host_device_vector.h"
 
 namespace xgboost {
 namespace common {
@@ -143,7 +144,7 @@ class ColumnSampler {
   */
   ColumnSampler() {
     uint32_t seed = common::GlobalRandom()();
-    rabit::Broadcast(&seed, sizeof(seed), 0);
+    collective::Broadcast(&seed, sizeof(seed), 0);
     rng_.seed(seed);
   }
 
@@ -156,9 +157,8 @@ class ColumnSampler {
    * \param colsample_bytree
    * \param skip_index_0      (Optional) True to skip index 0.
    */
-  void Init(int64_t num_col, std::vector<float> feature_weights,
-            float colsample_bynode, float colsample_bylevel,
-            float colsample_bytree, bool skip_index_0 = false) {
+  void Init(int64_t num_col, std::vector<float> feature_weights, float colsample_bynode,
+            float colsample_bylevel, float colsample_bytree) {
     feature_weights_ = std::move(feature_weights);
     colsample_bylevel_ = colsample_bylevel;
     colsample_bytree_ = colsample_bytree;
@@ -169,10 +169,8 @@ class ColumnSampler {
     }
     Reset();
 
-    int begin_idx = skip_index_0 ? 1 : 0;
-    feature_set_tree_->Resize(num_col - begin_idx);
-    std::iota(feature_set_tree_->HostVector().begin(),
-              feature_set_tree_->HostVector().end(), begin_idx);
+    feature_set_tree_->Resize(num_col);
+    std::iota(feature_set_tree_->HostVector().begin(), feature_set_tree_->HostVector().end(), 0);
 
     feature_set_tree_ = ColSample(feature_set_tree_, colsample_bytree_);
   }
