@@ -21,7 +21,7 @@ class ShotgunUpdater : public LinearUpdater {
       LOG(FATAL) << "Unsupported feature selector for shotgun updater.\n"
                  << "Supported options are: {cyclic, shuffle}";
     }
-    selector_.reset(FeatureSelector::Create(param_.feature_selector));
+    selector_.reset(FeatureSelector::Create(param_.feature_selector, ctx_->Threads()));
   }
   void LoadConfig(Json const& in) override {
     auto const& config = get<Object const>(in);
@@ -40,12 +40,13 @@ class ShotgunUpdater : public LinearUpdater {
 
     // update bias
     for (int gid = 0; gid < ngroup; ++gid) {
-      auto grad = GetBiasGradientParallel(gid, ngroup,
-                                          in_gpair->ConstHostVector(), p_fmat);
+      auto grad = GetBiasGradientParallel(gid, ngroup, in_gpair->ConstHostVector(), p_fmat,
+                                          ctx_->Threads());
       auto dbias = static_cast<bst_float>(param_.learning_rate *
                                CoordinateDeltaBias(grad.first, grad.second));
       model->Bias()[gid] += dbias;
-      UpdateBiasResidualParallel(gid, ngroup, dbias, &in_gpair->HostVector(), p_fmat);
+      UpdateBiasResidualParallel(gid, ngroup, dbias, &in_gpair->HostVector(), p_fmat,
+                                 ctx_->Threads());
     }
 
     // lock-free parallel updates of weights
@@ -54,42 +55,35 @@ class ShotgunUpdater : public LinearUpdater {
     for (const auto &batch : p_fmat->GetBatches<CSCPage>()) {
       auto page = batch.GetView();
       const auto nfeat = static_cast<bst_omp_uint>(batch.Size());
-      dmlc::OMPException exc;
-#pragma omp parallel for schedule(static)
-      for (bst_omp_uint i = 0; i < nfeat; ++i) {
-        exc.Run([&]() {
-          int ii = selector_->NextFeature
-            (i, *model, 0, in_gpair->ConstHostVector(), p_fmat, param_.reg_alpha_denorm,
-            param_.reg_lambda_denorm);
-          if (ii < 0) return;
-          const bst_uint fid = ii;
-          auto col = page[ii];
-          for (int gid = 0; gid < ngroup; ++gid) {
-            double sum_grad = 0.0, sum_hess = 0.0;
-            for (auto& c : col) {
-              const GradientPair &p = gpair[c.index * ngroup + gid];
-              if (p.GetHess() < 0.0f) continue;
-              const bst_float v = c.fvalue;
-              sum_grad += p.GetGrad() * v;
-              sum_hess += p.GetHess() * v * v;
-            }
-            bst_float &w = (*model)[fid][gid];
-            auto dw = static_cast<bst_float>(
-                param_.learning_rate *
-                CoordinateDelta(sum_grad, sum_hess, w, param_.reg_alpha_denorm,
-                                param_.reg_lambda_denorm));
-            if (dw == 0.f) continue;
-            w += dw;
-            // update grad values
-            for (auto& c : col) {
-              GradientPair &p = gpair[c.index * ngroup + gid];
-              if (p.GetHess() < 0.0f) continue;
-              p += GradientPair(p.GetHess() * c.fvalue * dw, 0);
-            }
+      common::ParallelFor(nfeat, ctx_->Threads(), [&](auto i) {
+        int ii = selector_->NextFeature(i, *model, 0, in_gpair->ConstHostVector(), p_fmat,
+                                        param_.reg_alpha_denorm, param_.reg_lambda_denorm);
+        if (ii < 0) return;
+        const bst_uint fid = ii;
+        auto col = page[ii];
+        for (int gid = 0; gid < ngroup; ++gid) {
+          double sum_grad = 0.0, sum_hess = 0.0;
+          for (auto &c : col) {
+            const GradientPair &p = gpair[c.index * ngroup + gid];
+            if (p.GetHess() < 0.0f) continue;
+            const bst_float v = c.fvalue;
+            sum_grad += p.GetGrad() * v;
+            sum_hess += p.GetHess() * v * v;
           }
-        });
-      }
-      exc.Rethrow();
+          bst_float &w = (*model)[fid][gid];
+          auto dw = static_cast<bst_float>(
+              param_.learning_rate * CoordinateDelta(sum_grad, sum_hess, w, param_.reg_alpha_denorm,
+                                                     param_.reg_lambda_denorm));
+          if (dw == 0.f) continue;
+          w += dw;
+          // update grad values
+          for (auto &c : col) {
+            GradientPair &p = gpair[c.index * ngroup + gid];
+            if (p.GetHess() < 0.0f) continue;
+            p += GradientPair(p.GetHess() * c.fvalue * dw, 0);
+          }
+        }
+      });
     }
   }
 
