@@ -16,7 +16,6 @@ namespace common {
 
 size_t GetNThreads() { return common::OmpGetNumThreads(0); }
 
-template <typename GradientSumT>
 void ParallelGHistBuilderReset() {
   constexpr size_t kBins = 10;
   constexpr size_t kNodes = 5;
@@ -25,29 +24,30 @@ void ParallelGHistBuilderReset() {
   constexpr double kValue = 1.0;
   const size_t nthreads = GetNThreads();
 
-  HistCollection<GradientSumT> collection;
+  HistCollection collection;
   collection.Init(kBins);
 
   for(size_t inode = 0; inode < kNodesExtended; inode++) {
     collection.AddHistRow(inode);
   }
   collection.AllocateAllData();
-  ParallelGHistBuilder<GradientSumT> hist_builder;
+  ParallelGHistBuilder hist_builder;
   hist_builder.Init(kBins);
-  std::vector<GHistRow<GradientSumT>> target_hist(kNodes);
+  std::vector<GHistRow> target_hist(kNodes);
   for(size_t i = 0; i < target_hist.size(); ++i) {
     target_hist[i] = collection[i];
   }
 
-  common::BlockedSpace2d space(kNodes, [&](size_t node) { return kTasksPerNode; }, 1);
+  common::BlockedSpace2d space(
+      kNodes, [&](size_t /* node*/) { return kTasksPerNode; }, 1);
   hist_builder.Reset(nthreads, kNodes, space, target_hist);
 
-  common::ParallelFor2d(space, nthreads, [&](size_t inode, common::Range1d r) {
+  common::ParallelFor2d(space, nthreads, [&](size_t inode, common::Range1d) {
     const size_t tid = omp_get_thread_num();
 
-    GHistRow<GradientSumT> hist = hist_builder.GetInitializedHist(tid, inode);
+    GHistRow hist = hist_builder.GetInitializedHist(tid, inode);
     // fill hist by some non-null values
-    for(size_t j = 0; j < kBins; ++j) {
+    for (size_t j = 0; j < kBins; ++j) {
       hist[j].Add(kValue, kValue);
     }
   });
@@ -57,23 +57,22 @@ void ParallelGHistBuilderReset() {
   for(size_t i = 0; i < target_hist.size(); ++i) {
     target_hist[i] = collection[i];
   }
-  common::BlockedSpace2d space2(kNodesExtended, [&](size_t node) { return kTasksPerNode; }, 1);
+  common::BlockedSpace2d space2(
+      kNodesExtended, [&](size_t /*node*/) { return kTasksPerNode; }, 1);
   hist_builder.Reset(nthreads, kNodesExtended, space2, target_hist);
 
-  common::ParallelFor2d(space2, nthreads, [&](size_t inode, common::Range1d r) {
+  common::ParallelFor2d(space2, nthreads, [&](size_t inode, common::Range1d) {
     const size_t tid = omp_get_thread_num();
 
-    GHistRow<GradientSumT> hist = hist_builder.GetInitializedHist(tid, inode);
+    GHistRow hist = hist_builder.GetInitializedHist(tid, inode);
     // fill hist by some non-null values
-    for(size_t j = 0; j < kBins; ++j) {
+    for (size_t j = 0; j < kBins; ++j) {
       ASSERT_EQ(0.0, hist[j].GetGrad());
       ASSERT_EQ(0.0, hist[j].GetHess());
     }
   });
 }
 
-
-template <typename GradientSumT>
 void ParallelGHistBuilderReduceHist(){
   constexpr size_t kBins = 10;
   constexpr size_t kNodes = 5;
@@ -81,28 +80,29 @@ void ParallelGHistBuilderReduceHist(){
   constexpr double kValue = 1.0;
   const size_t nthreads = GetNThreads();
 
-  HistCollection<GradientSumT> collection;
+  HistCollection collection;
   collection.Init(kBins);
 
   for(size_t inode = 0; inode < kNodes; inode++) {
     collection.AddHistRow(inode);
   }
   collection.AllocateAllData();
-  ParallelGHistBuilder<GradientSumT> hist_builder;
+  ParallelGHistBuilder hist_builder;
   hist_builder.Init(kBins);
-  std::vector<GHistRow<GradientSumT>> target_hist(kNodes);
+  std::vector<GHistRow> target_hist(kNodes);
   for(size_t i = 0; i < target_hist.size(); ++i) {
     target_hist[i] = collection[i];
   }
 
-  common::BlockedSpace2d space(kNodes, [&](size_t node) { return kTasksPerNode; }, 1);
+  common::BlockedSpace2d space(
+      kNodes, [&](size_t /*node*/) { return kTasksPerNode; }, 1);
   hist_builder.Reset(nthreads, kNodes, space, target_hist);
 
   // Simple analog of BuildHist function, works in parallel for both tree-nodes and data in node
-  common::ParallelFor2d(space, nthreads, [&](size_t inode, common::Range1d r) {
+  common::ParallelFor2d(space, nthreads, [&](size_t inode, common::Range1d) {
     const size_t tid = omp_get_thread_num();
 
-    GHistRow<GradientSumT> hist = hist_builder.GetInitializedHist(tid, inode);
+    GHistRow hist = hist_builder.GetInitializedHist(tid, inode);
     for(size_t i = 0; i < kBins; ++i) {
       hist[i].Add(kValue, kValue);
     }
@@ -120,21 +120,9 @@ void ParallelGHistBuilderReduceHist(){
   }
 }
 
-TEST(ParallelGHistBuilder, ResetDouble) {
-  ParallelGHistBuilderReset<double>();
-}
+TEST(ParallelGHistBuilder, Reset) { ParallelGHistBuilderReset(); }
 
-TEST(ParallelGHistBuilder, ResetFloat) {
-  ParallelGHistBuilderReset<float>();
-}
-
-TEST(ParallelGHistBuilder, ReduceHistDouble) {
-  ParallelGHistBuilderReduceHist<double>();
-}
-
-TEST(ParallelGHistBuilder, ReduceHistFloat) {
-  ParallelGHistBuilderReduceHist<float>();
-}
+TEST(ParallelGHistBuilder, ReduceHist) { ParallelGHistBuilderReduceHist(); }
 
 TEST(CutsBuilder, SearchGroupInd) {
   size_t constexpr kNumGroups = 4;
@@ -275,8 +263,7 @@ TEST(HistUtil, DenseCutsExternalMemory) {
   for (auto num_rows : sizes) {
     auto x = GenerateRandom(num_rows, num_columns);
     dmlc::TemporaryDirectory tmpdir;
-    auto dmat =
-        GetExternalMemoryDMatrixFromData(x, num_rows, num_columns, 50, tmpdir);
+    auto dmat = GetExternalMemoryDMatrixFromData(x, num_rows, num_columns, tmpdir);
     for (auto num_bins : bin_sizes) {
       HistogramCuts cuts = SketchOnDMatrix(dmat.get(), num_bins, common::OmpGetNumThreads(0));
       ValidateCuts(cuts, dmat.get(), num_bins);
