@@ -11,7 +11,7 @@
 
 #include "../../../src/tree/param.h"
 #include "../../../src/tree/split_evaluator.h"
-#include "../../../src/tree/updater_quantile_hist.h"
+#include "../../../src/tree/common_row_partitioner.h"
 #include "../helpers.h"
 #include "test_partitioner.h"
 #include "xgboost/data.h"
@@ -23,7 +23,7 @@ TEST(QuantileHist, Partitioner) {
   GenericParameter ctx;
   ctx.InitAllowUnknown(Args{});
 
-  HistRowPartitioner partitioner{n_samples, base_rowid, ctx.Threads()};
+  CommonRowPartitioner partitioner{&ctx, n_samples, base_rowid};
   ASSERT_EQ(partitioner.base_rowid, base_rowid);
   ASSERT_EQ(partitioner.Size(), 1);
   ASSERT_EQ(partitioner.Partitions()[0].Size(), n_samples);
@@ -34,15 +34,14 @@ TEST(QuantileHist, Partitioner) {
   auto cuts = common::SketchOnDMatrix(Xy.get(), 64, ctx.Threads());
 
   for (auto const& page : Xy->GetBatches<SparsePage>()) {
-    GHistIndexMatrix gmat;
-    gmat.Init(page, {}, cuts, 64, false, 0.5, ctx.Threads());
+    GHistIndexMatrix gmat(page, {}, cuts, 64, true, 0.5, ctx.Threads());
     bst_feature_t const split_ind = 0;
     common::ColumnMatrix column_indices;
-    column_indices.Init(page, gmat, 0.5, ctx.Threads());
+    column_indices.InitFromSparse(page, gmat, 0.5, ctx.Threads());
     {
       auto min_value = gmat.cut.MinValues()[split_ind];
       RegTree tree;
-      HistRowPartitioner partitioner{n_samples, base_rowid, ctx.Threads()};
+      CommonRowPartitioner partitioner{&ctx, n_samples, base_rowid};
       GetSplit(&tree, min_value, &candidates);
       partitioner.UpdatePosition<false, true>(&ctx, gmat, column_indices, candidates, &tree);
       ASSERT_EQ(partitioner.Size(), 3);
@@ -50,7 +49,7 @@ TEST(QuantileHist, Partitioner) {
       ASSERT_EQ(partitioner[2].Size(), n_samples);
     }
     {
-      HistRowPartitioner partitioner{n_samples, base_rowid, ctx.Threads()};
+      CommonRowPartitioner partitioner{&ctx, n_samples, base_rowid};
       auto ptr = gmat.cut.Ptrs()[split_ind + 1];
       float split_value = gmat.cut.Values().at(ptr / 2);
       RegTree tree;
