@@ -1,12 +1,12 @@
 import sys
 import numpy as np
 
-import unittest
 import pytest
 
 import xgboost as xgb
 sys.path.append("tests/python")
 import testing as tm
+import test_monotone_constraints as tmc
 
 rng = np.random.RandomState(1994)
 
@@ -30,19 +30,34 @@ def assert_constraint(constraint, tree_method):
     bst = xgb.train(param, dtrain)
     dpredict = xgb.DMatrix(X[X[:, 0].argsort()])
     pred = bst.predict(dpredict)
+
     if constraint > 0:
         assert non_decreasing(pred)
     elif constraint < 0:
         assert non_increasing(pred)
 
 
-class TestMonotonicConstraints(unittest.TestCase):
-    @pytest.mark.skipif(**tm.no_sklearn())
-    def test_exact(self):
-        assert_constraint(1, 'exact')
-        assert_constraint(-1, 'exact')
+@pytest.mark.skipif(**tm.no_sklearn())
+def test_gpu_hist_basic():
+    assert_constraint(1, 'gpu_hist')
+    assert_constraint(-1, 'gpu_hist')
 
-    @pytest.mark.skipif(**tm.no_sklearn())
-    def test_gpu_hist(self):
-        assert_constraint(1, 'gpu_hist')
-        assert_constraint(-1, 'gpu_hist')
+
+def test_gpu_hist_depthwise():
+    params = {
+        'tree_method': 'gpu_hist',
+        'grow_policy': 'depthwise',
+        'monotone_constraints': '(1, -1)'
+    }
+    model = xgb.train(params, tmc.training_dset)
+    tmc.is_correctly_constrained(model)
+
+
+def test_gpu_hist_lossguide():
+    params = {
+        'tree_method': 'gpu_hist',
+        'grow_policy': 'lossguide',
+        'monotone_constraints': '(1, -1)'
+    }
+    model = xgb.train(params, tmc.training_dset)
+    tmc.is_correctly_constrained(model)
